#Created on APRIL 2017                                                                                                                              
#@author: JC 
from gestalt import nodes
from gestalt import interfaces
from gestalt import machines
from gestalt import functions
from gestalt.machines import elements
from gestalt.machines import kinematics
from gestalt.machines import state
from gestalt.utilities import notice
from gestalt.publish import rpc    #remote procedure call dispatcher
import time
import io
import Tkinter as tk

size_bed_machine = 220.0
size_window = 500.0
scale_ratio = size_window/size_bed_machine
x_old = 0
y_old = 0
threshold = 0.1
machine_speed = 10

def motion(event):
    global x_old
    global y_old
    global threshold
    x, y = event.x, event.y
    x_machine = x/scale_ratio
    y_machine = y/scale_ratio
    print('window{}, {}'.format(x, y))
    print('machine:{}, {}'.format(x_machine, y_machine))
    if abs(x_machine - x_old) > threshold or abs(y_machine - y_old) > threshold:
        print('go')
        x_old = x_machine
        y_old = y_machine
        move = [-x_old, y_old] 
        stages.move(move, 0)
        status = stages.xAxisNode.spinStatusRequest()

class virtualMachine(machines.virtualMachine):

        
    def initInterfaces(self):
        if self.providedInterface: self.fabnet = self.providedInterface        #providedInterface is defined in the virtualMachine class.
        else: self.fabnet = interfaces.gestaltInterface('FABNET', interfaces.serialInterface(baudRate = 115200, interfaceType = 'ftdi', portName = '/dev/ttyUSB0'))
        
    def initControllers(self):
        self.xAxisNode = nodes.networkedGestaltNode('X Axis', self.fabnet, filename = '086-005a.py', persistence = self.persistence)
        self.yAxisNode = nodes.networkedGestaltNode('Y Axis', self.fabnet, filename = '086-005a.py', persistence = self.persistence)

        self.xyNode = nodes.compoundNode(self.xAxisNode, self.yAxisNode)

    def initCoordinates(self):
        self.position = state.coordinate(['mm', 'mm'])
    
    def initKinematics(self):
        self.xAxis = elements.elementChain.forward([elements.microstep.forward(4), elements.stepper.forward(1.8), elements.leadscrew.forward(8), elements.invert.forward(True)])
        self.yAxis = elements.elementChain.forward([elements.microstep.forward(4), elements.stepper.forward(1.8), elements.leadscrew.forward(8), elements.invert.forward(False)])        
        self.stageKinematics = kinematics.direct(2)    #direct drive on all axes
    
    def initFunctions(self):
        self.move = functions.move(virtualMachine = self, virtualNode = self.xyNode, axes = [self.xAxis, self.yAxis], kinematics = self.stageKinematics, machinePosition = self.position,planner = 'null')
        self.jog = functions.jog(self.move)    #an incremental wrapper for the move function
        pass
        
    def initLast(self):
        #self.machineControl.setMotorCurrents(aCurrent = 0.8, bCurrent = 0.8, cCurrent = 0.8)
        #self.xNode.setVelocityRequest(0)    #clear velocity on nodes. Eventually this will be put in the motion planner on initialization to match state.
        pass
    
    def publish(self):
        #self.publisher.addNodes(self.machineControl)
        pass
    
    def getPosition(self):
        return {'position':self.position.future()}
    
    def setPosition(self, position  = [None]):
        self.position.future.set(position)

    def setSpindleSpeed(self, speedFraction):
        #self.machineControl.pwmRequest(speedFraction)
        pass

#------IF RUN DIRECTLY FROM TERMINAL------
if __name__ == '__main__':
    stages = virtualMachine(persistenceFile = "examples.vmp")
    stages.xyNode.setVelocityRequest(machine_speed)   
    
    #move machine to initialize it
    stages.move([0.1, 0.1], 0)
    status = stages.xAxisNode.spinStatusRequest()
    
    # This checks to see if the move is done.
    while status['stepsRemaining'] > 0:
        time.sleep(0.001)
        status = stages.xAxisNode.spinStatusRequest()

    #global scale_ratio
    #global movement
    print('scale:{}',format(scale_ratio))
    root = tk.Tk()
    root.resizable(width=False, height=False)
    root.geometry('{}x{}'.format(500, 500))
    root.bind('<Motion>', motion)
    root.mainloop() #always end your script by calling the mainloop method of the root window.